package uk.me.piggott.simplechat;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;

public class ClientImpl implements Client {

	private ArrayList<String> messageList = new ArrayList<String>();
	private Server server;
	private String clientName;
	private Registry registry;
	private GUI gui;

	public String getName() {
		return clientName;
	}

	public ClientImpl(String clientName, GUI gui) throws RemoteException {
		this.clientName = clientName;
		this.gui = gui;
		UnicastRemoteObject.exportObject(this, 0);
	}

	public void connect(String serverAddress) throws NotBoundException, RemoteException {
		if(server != null) {
			server.unSubscribe(this);
		}
		registry = LocateRegistry.getRegistry(serverAddress, Registry.REGISTRY_PORT);
		server = (Server) registry.lookup("SimpleChat");
		server.subscribe(this);
	}

	public void disconnect() throws RemoteException, NotBoundException {
		server.unSubscribe(this);
		server = null;
	}

	public void sendMessage(String message) throws RemoteException {
		server.sendMessage(message);
	}

	public void update() throws RemoteException {
		int messagesSize = server.getMessagesSize();
		for(int i = messageList.size(); i < messagesSize; i++) {
			String message = server.getMessage(i);
			messageList.add(message);
			gui.showMessage(message);
		}
	}
}