package uk.me.piggott.simplechat;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.rmi.AlreadyBoundException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;

public class GUI extends JPanel implements ActionListener, WindowListener {

	private static final long serialVersionUID = 1L;
	private static final String VERSION = "Version 1.0.1";
	private JTextArea messageArea;
	private JTextField userInput;
	private JScrollPane messageAreaScrollPane;
	private JFrame frame;
	private Client client;
	private byte state = 0;
	
	/**
	 * States:
	 * 0 - Choosing server/client mode
	 * 5 - Entering client name
	 * 6 - Connecting to server
	 * 7 - Connected to server, chat in progress
	 * 10 - Running in server mode
	 */

	public GUI(JFrame frame) {
		super(new BorderLayout());
		this.frame = frame;
		Border etch = BorderFactory.createEtchedBorder(EtchedBorder.LOWERED);

		messageArea = new JTextArea("Type server to run a server, or client to start as a client:");
		messageArea.setEditable(false);
		messageArea.setLineWrap(true);

		messageAreaScrollPane = new JScrollPane(messageArea);
		messageAreaScrollPane.setPreferredSize(new Dimension(640, 480));
		messageAreaScrollPane.setBorder(etch);
		
		messageAreaScrollPane.getVerticalScrollBar().addAdjustmentListener(new AdjustmentListener(){
			public void adjustmentValueChanged(AdjustmentEvent e){
				messageArea.setCaretPosition(messageArea.getDocument().getLength());
			}});
		
		userInput = new JTextField();
		userInput.setBorder(etch);
		userInput.addActionListener(this);

		add(messageAreaScrollPane, BorderLayout.CENTER);
		add(userInput, BorderLayout.PAGE_END);

	}

	public void showMessage(String message) {
		messageArea.append("\n"+message);
	}
	
	public void actionPerformed(ActionEvent event) {
		String input = userInput.getText();
		userInput.setText("");
		switch (state) {
		case 0:
			messageArea.append("\n"+input);
			if (input.equals("client")) {
				frame.setTitle("SimpleChatClient "+GUI.VERSION);
				messageArea.append("\n\nEnter your name:");
				state = 5;
			} else if (input.equals("server")) {
		    	try {
					new ServerImpl(this);
					frame.setTitle("SimpleChatServer " +GUI.VERSION);
					state = 10;
				} catch (RemoteException e) {
					messageArea.append("\n\nSimpleChatServer initialisation error.");
				}
			} else {
				messageArea.append("\n\nType server to run a server, or client to start as a client:");
			}
			break;
		case 5:
			messageArea.append("\n"+input);
			try {
				client = new ClientImpl(input, this);
				messageArea.append("\n\nEnter the server address:");
				state++;
			} catch (RemoteException error) {
				messageArea.append("\nSimpleChatClient initialisation error.");
			}
			break;
		case 6:
			try {
				messageArea.append("\n"+input);
				messageArea.append("\n\nConnecting to "+input+"...");
				client.connect(input);
				frame.setTitle("SimpleChatClient "+GUI.VERSION+" - Connected to "+input);
				state++;
			} catch (RemoteException error) {
				messageArea.append("\nError connecting to "+input+".");
				messageArea.append("\nEnter the server address:");
			} catch (AlreadyBoundException error) {
				messageArea.append("\nError binding to "+input+".");
				messageArea.append("\nEnter the server address:");
			} catch (NotBoundException error) {
				messageArea.append("\nError binding to "+input+".");
				messageArea.append("\nEnter the server address:");
			}
			break;
		case 7:
			if (!input.equals("")) {
				try {
					client.sendMessage(client.getName()+": "+input);
				} catch (RemoteException error) {
					messageArea.append("\nError communicating with server.");
					userInput.setText(input);
				}
			}
			break;
		}
	}

	public void windowActivated(WindowEvent arg0) {}
	public void windowClosed(WindowEvent arg0) {}
	public void windowClosing(WindowEvent arg0) {
		switch (state) {
		case 7:
			try {
				client.disconnect();
			} catch (RemoteException e) {
			} catch (NotBoundException e) {
			}
			System.exit(0);
			break;
		default:
			System.exit(0);
		break;
		}
	}
	public void windowDeactivated(WindowEvent arg0) {}
	public void windowDeiconified(WindowEvent arg0) {}
	public void windowIconified(WindowEvent arg0) {}
	public void windowOpened(WindowEvent arg0) {
		userInput.requestFocusInWindow();
	}

	public static void main(String[] args) {
	    try {
	        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	    } 
	    catch (UnsupportedLookAndFeelException e) {}
	    catch (ClassNotFoundException e) {}
	    catch (InstantiationException e) {}
	    catch (IllegalAccessException e) {}
	    
		//Schedule a job for the event dispatch thread: creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				JFrame frame = new JFrame("SimpleChat "+GUI.VERSION);
				GUI gui = new GUI(frame);
				frame.add(gui);
				frame.addWindowListener(gui);
				frame.pack();
				frame.setVisible(true);
			}
		});
	}
}